# -*- coding: utf-8 -*-
'''
Created on 15.06.2010

@author: adm
'''

from datetime import date, datetime, time, timedelta
from calendar import monthrange
from dateutil.relativedelta import relativedelta
import logging

log = logging.getLogger("Campaign")


class DiscountCampaignGroup(object):
    """
    Group of campaign
    """
    priority = None

    def __init__(self, interactionType=None, childGroups=None, discountCampaigns=None, id=None, name=None, parentGroup=None):
        """
        interactionType - type of interaction . InteractionType (ALL, FIRST, LAST, MAX, MIN)
        childGroups[] - list of child DiscountCampaignGroup
        discountCampaigns[] - list of DiscountCampaign
        """
        self.interactionType = interactionType
        self.childGroups = childGroups
        if self.childGroups is None:
            self.childGroups = []
        self.discountCampaigns = discountCampaigns
        self.name = name
        self.id = id
        self.parentGroup = parentGroup
        self.priority = None

    def __repr__(self):
        return "<DiscountCampaignGroup(%s, '%s','%s','%s', '%s')>" % (self.id, repr(self.name), repr(self.interactionType), self.discountCampaigns, self.childGroups)


class DiscountCampaign(object):
    """
    Акция
    """
    weight = None
    beginBonusesIntervalCondition = None
    beginBonusesIntervalValue = None
    beginDateWithBeginNextMetric = None
    beginBonusesIntervalMetric = None
    endBonusesIntervalMetric = None
    endBonusesIntervalCondition = None
    endBonusesIntervalValue = None
    beginBonusesDateCalculate = None
    endBonusesDateCalculate = None
    beginRangeDate = None
    endRangeDate = None
    assetBeginDate = None
    assetEndDate = None
    beginTime = None
    endTime = None
    daysOfWeek = None
    manualActivate = None
    discountSystem = None
    priority = None
    # спецификаторы, позволяют фильтровать акции на уровне кассы и точки входа
    qualifiers = set()

    def __init__(self, parentGroup=None, interactionType=None, name=None, beginDate=None, endDate=None, discounts=None, id=None, active=True):
        """
        parentGroup - parent group. DiscountCampaignGroup
        interactionType - type of interaction. InteractionType (ALL, FIRST, LAST, MAX, MIN)
        name - name
        beginDate - begin date
        endDate - end date
        discounts[] - list of Discount
        discountListReverse - False - порядок элементов в discounts[] на обратный не меняли. True - порядок поменяли
        labels - список произвольных меток у акции ["aaa", "bbb"]
        """
        self.parentGroup = parentGroup
        self.name = name
        self.beginDate = beginDate
        self.endDate = endDate
        self.discounts = discounts if discounts else []
        self.interactionType = interactionType
        self.id = id
        self.active = active
        self.discountListReverse = False
        self.checkedTags = None
        self.tagsIsAllowed = True
        self.tagsValid = None
        self.labels = []
        self.discountSystem = 1
        self.priority = None
        self.qualifiers = set()

    def isActive(self):
        return self.active

    def isDiscountListReverse(self):
        """
        Изменен ли порядок элементов в discounts[] на обратный. 
        True - изменен, False - нет.
        """
        return self.discountReverse if hasattr(self, 'discountReverse') else False

    def setDiscountListInReverse(self):
        """
        Устанавливает признак того, что элементы в discounts[] располагаются в обратном порядке. 
        """
        self.discounts.reverse()
        self.discountReverse = True

    def getCheckedTags(self):
        return self.checkedTags if hasattr(self, 'checkedTags') else None

    def setCheckedTags(self, checkedTags):
        """
        Используется для тестов
        """
        self.checkedTags = checkedTags

    def isTagsAllowed(self):
        return self.tagsIsAllowed if hasattr(self, 'tagsIsAllowed') else True

    def setTagsAllowed(self, tagsIsAllowed):
        self.tagsIsAllowed = tagsIsAllowed

    def isTagsValid(self):
        return self.tagsValid if hasattr(self, 'tagsValid') else None

    def setTagsValid(self, tagsValid):
        self.tagsValid = tagsValid    

    def isValidTimePeriod(self, checkDateTime):
        """
        Проверяем, что акция действует в указанное время
        """
        if self.beginTime is None and self.endTime is None:
            return True

        bTime = time(0, 0, 0) if self.beginTime is None else datetime.strptime(self.beginTime, '%H:%M').time()
        eTime = time(23, 59, 59) if self.endTime is None else datetime.strptime(self.endTime, '%H:%M').time()

        if bTime < eTime:
            return checkDateTime.time() >= bTime and checkDateTime.time() <= eTime

        isTimeValid = checkDateTime.time() <= eTime or checkDateTime.time() >= bTime
        if self.beginDate is not None and self.endDate is not None:
            isTimeValid = isTimeValid and checkDateTime >= datetime.combine(self.beginDate, eTime) \
                    and checkDateTime <= datetime.combine(self.endDate, bTime)

        return isTimeValid

    def isValidDate(self, checkDateTime):
        """
        Validate current date
        return true if current data in range 
        """
        return (not self.daysOfWeek or checkDateTime.isoweekday() in self.daysOfWeek) \
            and (self.beginDate is None or checkDateTime.date() >= self.beginDate) \
            and (self.endDate is None or checkDateTime.date() <= self.endDate) \
            and self.isValidTimePeriod(checkDateTime)

    def isValidTag(self, tags):
        """
        Проверка того, что метки,
        к которым принадлежит внешний объект (напр. магазин),
        подходят к данной акции
        """
        if len(self.getCheckedTags()) == 0:
            self.setTagsValid(not self.isTagsAllowed())
        else:
            log.debug(u'Для меток %s акция %s ', self.getCheckedTags(),
                      u"разрешена" if self.isTagsAllowed() else u"запрещена")
            log.debug(u'Метки магазина: %s', tags)
            checkResult = len(self.getCheckedTags()) == len(list(
                filter(lambda item: set(item) & set(tags), self.getCheckedTags())))
            self.setTagsValid(self.isTagsAllowed())
            if not checkResult:
                self.setTagsValid(not self.isTagsAllowed())
            log.debug(u'Метки проверены и %s', u"действительны" if self.isTagsValid() else u"недействительны")
        return self.isTagsValid()

    def __repr__(self):
        return "<DicountCampaign(%s, '%s')>" % (self.id, repr(self.name))

    def _addMonths(self, sourcedate, months):
        month = sourcedate.month - 1 + months
        year = sourcedate.year + month // 12
        month = month % 12 + 1
        return date(year, month, sourcedate.day) if sourcedate.day <= monthrange(year, month)[1] \
                else date(year, month, monthrange(year, month)[1]) + timedelta(days=1)

    def _modifiedAddedTime(self, sourceDate, value, metric):
        result = date(sourceDate.year, sourceDate.month, sourceDate.day)
        if metric == "day":
            result = result + timedelta(days=value)
        elif metric == "week":
            result = result + timedelta(weeks=value)
        elif metric == "month":
            result = self._addMonths(result, value)
        return date(result.year, result.month, result.day)

    def _getCalculatedDate(self, calcExpression, check=None):
        """
        Расчет даты по выражению
        """
        rezDate = None
        try:
            rezDate = eval(calcExpression)
        except Exception as e:
            log.error(u'Не могу посчитать дату: %s' % e)
            rezDate = None
        if not (type(rezDate) in [date, datetime]):
            log.error(u'Результат не является объектом Date: %s' % type(rezDate))
            rezDate = None
        return rezDate

    def calcAssetDates(self, check=None):
        if not self.beginBonusesIntervalCondition or not self.endBonusesIntervalCondition:
            return

        tmpbegin = None
        tmpend = None

        if self.beginBonusesIntervalCondition == "through":
            tmpbegin = self._modifiedAddedTime(date.today(), self.beginBonusesIntervalValue, self.beginBonusesIntervalMetric)
        elif self.beginBonusesIntervalCondition == "date":
            tmpbegin = self.beginRangeDate
        elif self.beginBonusesIntervalCondition in ["begin_next", "begin_current"]:
            added = 1 if self.beginBonusesIntervalCondition == "begin_next" else 0
            metric = self.beginDateWithBeginNextMetric if self.beginBonusesIntervalCondition == "begin_next" else self.beginDateWithBeginCurrentMetric
            if metric == "day":
                tmpbegin = date.today() + timedelta(days=added)
            elif metric == "week":
                tmpdate = date.today() + timedelta(weeks=added)
                tmpdate = tmpdate - timedelta(days=tmpdate.weekday())
                tmpbegin = tmpdate
            elif metric == "month":
                tmpdate = date.today()
                tmpdate = tmpdate.replace(day=1)
                tmpbegin = self._addMonths(tmpdate, added)
        elif self.beginBonusesIntervalCondition == "calculate":
            tmpbegin = self._getCalculatedDate(self.beginBonusesDateCalculate, check.toDict())

        if self.endBonusesIntervalCondition == "through":
            tmpend = None if tmpbegin is None else self._modifiedAddedTime(tmpbegin, self.endBonusesIntervalValue, self.endBonusesIntervalMetric)
        elif self.endBonusesIntervalCondition == "date":
            tmpend = self.endRangeDate
        elif self.endBonusesIntervalCondition == "calculate":
            tmpend = self._getCalculatedDate(self.endBonusesDateCalculate, check.toDict())

        self.assetBeginDate = None if tmpbegin is None else tmpbegin.strftime("%Y-%m-%d")
        self.assetEndDate = None if tmpend is None else tmpend.strftime("%Y-%m-%d")
