# -*- coding: utf-8 -*-
'''
Created on 11.10.2011

@author: ays
'''

from artixds.conditionfunctionplugins.conditionplugin import add_plugin
from artixds.generalfunction.checkbycodecondition import inventQuantInCheckByCode
from artixds.generalfunction.conditionhelper import __timeIn, __dateIn, __getAsList, __hasOptions, __hasCatalogs,__inventInCheckByOption, __hasGroups
from artixds.generalfunction.conditionhelper import __isPartOfDisKit, __inventInCheck, __aggregateInventInCheck

@add_plugin
def checkTimeIn(self, beginTime = None, endTime = None):
    return __timeIn(self.check["datetimebeg"], beginTime, endTime)
    
@add_plugin
def checkInTime(self, beginTime = None, endTime = None):
    return __timeIn(self.check["datetimebeg"], beginTime, endTime)

@add_plugin
def checkDateIn(self, beginDate = None, endDate = None):
    return __dateIn(self.check["datetimebeg"].date(), beginDate, endDate)

@add_plugin
def checkWeekDayIn(self, weekDays):
    _weekDays = __getAsList(weekDays)
    return self.check["datetimebeg"].date().dayOfWeek() in _weekDays
    
@add_plugin
def weekDayIn(self, weekDays):
    _weekDays = __getAsList(weekDays)
    return self.check["datetimebeg"].date().dayOfWeek() in _weekDays

@add_plugin
def monthDayIn(self, daysList):
    day = self.check["datetimebeg"].date().day()
    return day in __getAsList(daysList)

@add_plugin
def inventInCheckByCode(self, code = None):
    return True if __inventInCheck(self.checkWrapper, "code", code) else False
    
@add_plugin
def inventInCheckByBcode(self, bcode = None):
    return True if __inventInCheck(self.checkWrapper, "bcode", bcode) else False

@add_plugin
def hasInventOptions(self, options = None):
    return __hasOptions(options, self.object["extendetoptions"])

@add_plugin   
def hasInventCatalogs(self, catalogs):
    return __hasCatalogs (catalogs, self.object["tmc"]["catalogs"])

@add_plugin
def hasInventGroup( self, groups):
    return __hasGroups(groups, self.object["tmc"]["tmcgroups"])

@add_plugin
def hasInventAllCatalogs( self, catalogs):
    return __hasCatalogs (catalogs, self.object["tmc"]["catalogs"],True)

@add_plugin
def inventInCheckByOption(self, options):
    return True if __inventInCheckByOption(self.checkWrapper, options) else None

@add_plugin
def isPartOfDisKit(self):
    return __isPartOfDisKit(self.object)
#    return "partOfKit" in self.object and self.object["partOfKit"] == True

@add_plugin
def isDiscountApplied(self):
    return "discountApplied" in self.object and self.object["discountApplied"] == True
    
@add_plugin
def hasInventDiscount(self):
    return "hasDiscount" in self.object and self.object["hasDiscount"] == True

@add_plugin
def hasInventBonus(self):
    return "hasBonus" in self.object and self.object["hasBonus"] == True

@add_plugin
def hasInventImpact(self):
    return "hasImpact" in self.object and self.object["hasImpact"] == True

@add_plugin
def hasBackInSalePosition(self):
    '''
    Определяет есть ли в чеке позиция 'Возврат в чеке продажи'.
    '''
    return 54 in [ item['opcode'] for item in self.check['goodsitems'] ]

@add_plugin
def noCardInCheck(self):
    '''
    Определяет отсутствуют ли в чеке карты.
    '''
    return len(self.check['cardrecords']) == 0

@add_plugin
def noCouponInCheck(self):
    '''
    Определяет отсутствуют ли в чеке купоны.
    '''
    return len(self.check['coupons']) == 0

@add_plugin
def couponNumberStartsWith(self, number=None):
    '''
    Определяет начинается ли номер купона с номера number.
    '''
    if not number:
        return False
    return self.coupon["number"].startswith(str(number))
    DONT_CACHE #Нужно чтобы избежать проблем с кэшированием

@add_plugin
def hasCampaignLabels(self, labels=[]):
    '''
    Определяет есть ли у позиции список меток
    '''
    if not labels:
        return False

    if not "campaignlabels" in self.object:
        return False

    return set(labels).issubset(self.object["campaignlabels"])

@add_plugin
def hasInventAllCodes(self, codes, requiredCount=1):
    '''
    Проверяет наличие в чеке всех товаров с кодами codes
    в количестве не менее requiredCount, по умолчанию 1
    '''

    if len(codes) == 0 or requiredCount < 1:
        return False

    for code in codes:
        if inventQuantInCheckByCode(self, code) < requiredCount:
            return False

    return True
