# -*- coding: utf-8 -*-

from enum import Enum
from artixds.campaign import DiscountCampaignGroup, DiscountCampaign


class DiscountSystemType(Enum):
    DISCOUNT = 'discount'
    BONUS = 'bonus'
    BONUSPAY = 'bonuspay'


class DiscountSystem(object):
    def __init__(self, rootCampaignGroup=None, type=DiscountSystemType.DISCOUNT.value, manualCampaigns=[], number=1, groupsAndCampaignsPriorities=[]):
        self.rootCampaignGroup = rootCampaignGroup
        self.type = type
        self.manualCampaigns = manualCampaigns
        self.number = number
        self.groupsAndCampaignsPriorities = groupsAndCampaignsPriorities
        self.qualifiers = set()

    def loadAdditionalInfo(self):
        """ Загрузка дополнительных данных из дерева дисконта: ручные скидки, приоритеты акций и квалификаторы """
        priority = None
        if hasattr(self.rootCampaignGroup, 'priority'):
            priority = self.rootCampaignGroup.priority

        self.manualCampaigns, priorities, self.qualifiers = self.__treeTraversal(self.rootCampaignGroup, priority)

        priorities = list(priorities)
        priorities.sort(key=lambda priority: -1 if priority is None else priority)
        if not priorities:
            priorities.append(None)
        self.groupsAndCampaignsPriorities = priorities

    def __treeTraversal(self, campaignGroup, parentPriority):
        """ Обход дерева дисконта и извлечение из него нужных данных """
        objects = campaignGroup.discountCampaigns + campaignGroup.childGroups
        manualCampaigns = []
        priorities = set()
        qualifiers = set()
        for obj in objects:
            if isinstance(obj, DiscountCampaign):
                if hasattr(obj, 'manualActivate') and obj.manualActivate and obj.isActive():
                    manualCampaigns.append(DiscountSystem.__createManualCampaign(obj))
                if hasattr(obj, 'priority'):
                    if obj.priority is None and parentPriority is not None:
                        obj.priority = parentPriority
                    priorities.add(obj.priority)
                if hasattr(obj, 'qualifiers'):
                    qualifiers = qualifiers.union(set(obj.qualifiers))
            elif isinstance(obj, DiscountCampaignGroup):
                groupPriority = parentPriority
                if hasattr(obj, 'priority'):
                    if obj.priority is not None:
                        groupPriority = obj.priority
                    else:
                        obj.priority = parentPriority
                mc, campaignPriorities, campaignQualifiers = self.__treeTraversal(obj, groupPriority)
                manualCampaigns.extend(mc)
                priorities = priorities.union(campaignPriorities)
                qualifiers = qualifiers.union(campaignQualifiers)
        return manualCampaigns, priorities, qualifiers

    @staticmethod
    def __createManualCampaign(campaign):
        """ Создание словаря ручной скидки из акции """
        campaignInfo = {
            'id': campaign.id,
            'name': campaign.name,
            'beginDate': campaign.beginDate,
            'endDate': campaign.endDate,
            'beginTime': campaign.beginTime,
            'endTime': campaign.endTime,
            'daysOfWeek': campaign.daysOfWeek,
            'checkedTags': campaign.checkedTags,
            'tagsIsAllowed': campaign.tagsIsAllowed
        }
        if campaign.discounts:
            campaignInfo['discountId'] = campaign.discounts[0].id
        return campaignInfo
