# -*- coding: utf-8 -*-
from artixds.ratefunctionplugins.extendedoption import ExtendedOptions

"""
Функции cheapGoodsWithOptions, cheapGoodsFromCatalog, cheapGoods разворачивают чек на позиции, 
выбирают все позиции:
cheapGoodsWithOptions - товары с опциями extops,
cheapGoodsFromCatalog - товары, входящие в каталоги extops,
cheapGoods - товары в чеке
сортируют эти позиции по цене, выбирают countDisc из count самых дешевых товаров
и рассчитывают скидку на эти товары. Скидка может раскидываться на все товары,
которые участвуют в акции или только на самые дешевые.
Возвращают словарь из номеров позиций в качестве ключей
и суммы скидки в качестве значений

В Юки:
    условие object["posnum"] in cf.cheapGoodsWithOptions(5, [u"num"], 50, 2)
    ставка rf.cheapGoodsWithOptions(5, [u"num"], 50, 2)[object["posnum"]]
"""

def distributeDiscount(items, itemsSum, count, rate, countDisc, toCheapest):
    sortedItems = sorted(items, key = lambda x: x[1]) # сортируем по цене
    # countDisc из count самых дешевых товаров
    cheapestItems = sortedItems[:(len(sortedItems) // count) * countDisc]
    result = {}
    if not cheapestItems:
        return result

    disc = rate / 100.0 # скидка
    # Сумма countDisc из count самых дешевых товаров
    cheapestSum = 0.0
    for item in cheapestItems:
        cheapestSum += item[1]
    discSum = cheapestSum * disc

    # Распределить на самые дешевые или на все с опциями
    if toCheapest:
        items = cheapestItems
    else:
        disc = discSum / itemsSum

    result = {}
    for item in items:
        if not item[0] in result:
            result[item[0]] = 0.0
        roundSum = round(min(item[1] * disc, discSum), 2)
        result[item[0]] += roundSum
        discSum -= roundSum

    if discSum > 0.005:
        result[items[-1][0]] += round(discSum, 2)

    return result


def splitItems(item, items):
    itemsSum = 0.0
    for _ in range(1 if item.data['tmcunit']['fractional'] else int(item.data['bquant'])):
        items.append((item.data['posnum'], item.data['price']))
        itemsSum += item.data['price']
    return itemsSum


"""
count - каждый N-ный
extops - опции
rate - скидка в процентах
countDisc - кол-во товаров из count, к которым применяется скидка
toCheapest - если true применяется скидка к самым дешевым, иначе ко всем товарам с опциями
"""
def cheapGoodsWithOptions(self, count, extops, rate, countDisc, toCheapest):
    items = [] # (позиция, цена), кол-во = 1
    itemsSum = 0.0 # сумма всех позиций с опциями
    for item in self.checkWrapper.getGoodsList():
        extendedOptions = ExtendedOptions(item.data["extendetoptions"])
        for extop in extops:
            if extendedOptions.hasOptionLike(extop) and item.data["price"] > 0.001:
                itemsSum += splitItems(item, items)
                break

    return distributeDiscount(items, itemsSum, count, rate, countDisc, toCheapest)

"""
count - каждый N-ный
extops - список каталогов
rate - скидка в процентах
countDisc - кол-во товаров из count, к которым применяется скидка
toCheapest - если true применяется скидка к самым дешевым, иначе ко всем товарам с опциями
"""
def cheapGoodsFromCatalog(self, count, extops, rate, countDisc, toCheapest):
    items = []
    itemsSum = 0.0
    catalogs = set(extops)
    for item in self.checkWrapper.getGoodsList():
        itemCatalogs = set(int(catalog["catalogid"]) for catalog in item.data["tmc"]["catalogs"])
        if itemCatalogs & catalogs and item.data["price"] > 0.001:
            itemsSum += splitItems(item, items)

    return distributeDiscount(items, itemsSum, count, rate, countDisc, toCheapest)

"""
count - каждый N-ный
rate - скидка в процентах
countDisc - кол-во товаров из count, к которым применяется скидка
toCheapest - если true применяется скидка к самым дешевым, иначе ко всем товарам
"""
def cheapGoods(self, count, rate, countDisc, toCheapest):
    items = []
    itemsSum = 0.0
    for item in self.checkWrapper.getGoodsList():
        if item.data["price"] > 0.001:
            itemsSum += splitItems(item, items)

    return distributeDiscount(items, itemsSum, count, rate, countDisc, toCheapest)

def cheapGoodsBySetWithConditions(self, rate, setSize, countDisc, conditions):
    """
    rate - скидка в процентах
    setSize - кол-во товаров, из которых формируются наборы в чеке;
    countDisc - кол-во товаров в наборе, на которые применяется скидка
    conditions - условия скидки
    """
    items = []
    conditions = __import__('artixds.discountcondition', fromlist='q').DiscountCondition(condition=conditions)
    conditions.checkWrapper = self.checkWrapper
    for item in self.checkWrapper.getGoodsList():
        if conditions.validate(check=self.checkWrapper.checkDict, object=item.data):
            for _ in range(int(item.data['bquant'])):
                items.append(item)

    #Сортируем от дорогих к дешёвым
    items.sort(reverse=True, key=lambda position: position.data['price'])

    discount = {}
    for i in range(0, len(items) // setSize * setSize, setSize):
        summ = 0.0
        discountSumm = 0.0

        nextStep = i + setSize
        for discounted in range(i + setSize - countDisc, nextStep):
            discountSumm += items[discounted].data['price'] * rate * 0.01

        discountRemain = discountSumm
        for item in range(i, nextStep):
            summ += items[item].data['price']

        posNum = 0
        for item in range(i, nextStep):
            posNum = items[item].data['posnum']
            roundSum = round(min(items[item].data['price'] / summ * discountSumm, discountRemain), 2)
            discountRemain -= roundSum
            discount[posNum] = discount.get(posNum, 0.0) + roundSum

        if discountRemain >= 0.005:
            discount[posNum] += round(discountRemain, 2)

    return discount