# -*- coding: utf-8 -*-

from artixds.generalfunction.conditionhelper import __hasGroups, __hasCatalogs


def countersQuantIndexPrice(self, idCounter, limit, indexPrice, conditionProperties, calcWhenLessLimit,
                            alwaysCalcDiscount):
    """
    Применение индексной цены с условием по сумме счетчика
    :param idCounter идентификатор счетчика
    :param limit: пороговое значение для счетчика
    :param indexPrice: индекс цены
    :param conditionProperties: словарь со свойствами для проверки { "prop1": "value, "prop2": ["value1", "value2] }
    :param calcWhenLessLimit: давать скидку при значении счетчика limit, иначе от limit
    :param alwaysCalcDiscount: давать скидку, если от сервера не получены счетчики
    :return словарь из списка позиции и суммы скидки
    """

    indexPrice = str(indexPrice)
    counterSum = getCounterSum(self.check, idCounter, limit, calcWhenLessLimit, alwaysCalcDiscount)
    if counterSum == -1:
        return {}

    positions = {}
    totals = counterSum

    for item in self.checkWrapper.getGoodsList():
        properties = conditionProperties.keys()
        condition = True
        for prop in properties:
            allowedValues = conditionProperties[prop]

            if not isinstance(allowedValues, list):
                tmp = allowedValues
                allowedValues = []
                allowedValues.append(str(tmp))
 
            if prop == "catalogs":
                if not __hasCatalogs(allowedValues, item.data["tmc"]["catalogs"]):
                    condition = False
                    break
                else:
                    continue

            if prop == "groups":
                if not __hasGroups(allowedValues, item.data["tmc"]["tmcgroups"]):
                    condition = False
                    break
                else:
                    continue

            if str(item.data[prop]) not in allowedValues:
                condition = False
                break

        if not condition:
            continue

        if calcWhenLessLimit and (totals < limit):
            count = min(item.data['bquant'], limit - totals)
            totals += count
        elif not calcWhenLessLimit:
            if (item.data['bquant'] + totals) > limit:
                count = min(item.data['bquant'] + totals - limit, item.data['bquant'])
                totals += item.data['bquant']
            else:
                totals += item.data['bquant']
                continue
        else:
            continue

        if indexPrice in item.data['tmc']['indexprices']:
            newPrice = round(item.data['tmc']['indexprices'][indexPrice]['price'] * count, 2)
            currentPrice = round(item.data['price'] * count, 2)
            discountSum = currentPrice - newPrice
            if discountSum > 0:
                positions[item.data['posnum']] = discountSum

    return positions


def countersQuantByBcode(self, idCounter, bcode, percent, limit, calcWhenLessLimit, alwaysCalcDiscount):

    counterSum = getCounterSum(self.check, idCounter, limit, calcWhenLessLimit, alwaysCalcDiscount)
    if counterSum == -1:
        return {}

    positions = {}
    totals = counterSum

    for item in self.checkWrapper.getGoodsList():
        itemBcode = item.data['bcode']

        if itemBcode not in bcode:
            continue

        if calcWhenLessLimit and (totals < limit):
            count = min(item.data['bquant'], limit - totals)
            totals += count
        elif not calcWhenLessLimit:
            if (item.data['bquant'] + totals) > limit:
                count = min(item.data['bquant'] + totals - limit, item.data['bquant'])
                totals += item.data['bquant']
            else:
                totals += item.data['bquant']
                continue
        else:
            continue

        positions[item.data['posnum']] = item.data['price'] * percent / 100.0 * count

    return positions


def countersQuantByCode(self, idCounter, code, percent, limit, calcWhenLessLimit, alwaysCalcDiscount):

    counterSum = getCounterSum(self.check, idCounter, limit, calcWhenLessLimit, alwaysCalcDiscount)
    if counterSum == -1:
        return {}

    positions = {}
    totals = counterSum
    count = 0

    for item in self.checkWrapper.getGoodsList():

        if item.data['code'] not in code:
            continue

        if calcWhenLessLimit and (totals < limit):
            count = min(item.data['bquant'], limit - totals)
            totals += count
        elif not calcWhenLessLimit:
            if (item.data['bquant'] + totals) > limit:
                count = min(item.data['bquant'] + totals - limit, item.data['bquant'])
                totals += item.data['bquant']
            else:
                totals += item.data['bquant']
                continue
        else:
            continue

        positions[item.data['posnum']] = item.data['price'] * percent / 100.0 * count

    return positions


def countersQuantByCatalog(self, idCounter, catalog, percent, limit, calcWhenLessLimit, alwaysCalcDiscount):

    counterSum = getCounterSum(self.check, idCounter, limit, calcWhenLessLimit, alwaysCalcDiscount)
    if counterSum == -1:
        return {}

    positions = {}
    totals = counterSum

    for item in self.checkWrapper.getGoodsList():

        if not __hasCatalogs(catalog, item.data["tmc"]["catalogs"]):
            continue

        if calcWhenLessLimit and (totals < limit):
            count = min(item.data['bquant'], limit - totals)
            totals += count
        elif not calcWhenLessLimit:
            if (item.data['bquant'] + totals) > limit:
                count = min(item.data['bquant'] + totals - limit, item.data['bquant'])
                totals += item.data['bquant']
            else:
                totals += item.data['bquant']
                continue
        else:
            continue

        positions[item.data['posnum']] = item.data['price'] * percent / 100.0 * count

    return positions


def countersQuantByGroup(self, idCounter, group, percent, limit, calcWhenLessLimit, alwaysCalcDiscount):

    counterSum = getCounterSum(self.check, idCounter, limit, calcWhenLessLimit, alwaysCalcDiscount)
    if counterSum == -1:
        return {}

    positions = {}
    totals = counterSum

    for item in self.checkWrapper.getGoodsList():

        if not __hasGroups(group, item.data["tmc"]["tmcgroups"]):
            continue

        if calcWhenLessLimit and (totals < limit):
            count = min(item.data['bquant'], limit - totals)
            totals += count
        elif not calcWhenLessLimit:
            if (item.data['bquant'] + totals) > limit:
                count = min(item.data['bquant'] + totals - limit, item.data['bquant'])
                totals += item.data['bquant']
            else:
                totals += item.data['bquant']
                continue
        else:
            continue

        positions[item.data['posnum']] = item.data['price'] * percent / 100.0 * count

    return positions


def countersQuantByAccumulation(self, idCounter, ranges, defaultIndex):

    if self.check.get("countersstatus", 0) == 1:
        return defaultIndex
    elif self.check.get("countersstatus", 0) != 2:
        return -1

    counterValue = self.check.get("counters", {}).get(idCounter.lower(), 0)

    index = 0
    for r in ranges:
        index += 1
        if not isinstance(r, tuple) or len(r) != 2:
            continue

        if r[0] <= counterValue and (not r[1] or counterValue <= r[1]):
            return index
    return -1


def getCounterSum(check, idCounter, limit, calcWhenLessLimit, alwaysCalcDiscount):
    """
    Получение значения счетчика из чека
    :param idCounter идентификатор счетчика
    :param limit: пороговое значение для счетчика
    :param calcWhenLessLimit: давать скидку при значении счетчика limit, иначе от limit
    :param alwaysCalcDiscount: давать скидку, если от сервера не получены счетчики
    :return: значение счетика, -1 если условие не валидно
    """
    counterSum = -1

    if check.get("countersstatus", 0) == 2:
        counterSum = check.get("counters", {}).get(idCounter.lower(), 0)
    elif check.get("countersstatus", 0) == 1:
        if alwaysCalcDiscount:
            if calcWhenLessLimit:
                counterSum = 0
            else:
                counterSum = limit

    return counterSum
