# -*- coding: utf-8 -*-
from artixds.ratefunctionplugins.extendedoption import ExtendedOptions

def calcPieceGoodsDiscount(self, pieceGoods, totalQuant, params):
    '''
    Функция вычисления скидок для штучных товаров
    '''
    result = {}
    for bcode in pieceGoods:
        limit = min(totalQuant[bcode] // params[bcode][0], params[bcode][1])
        for (pos, quant, price) in pieceGoods[bcode]:
            if limit <= 0:
                break
            elif limit <= quant:
                result[pos] = limit * price * params[bcode][2] / 100.0
                break
            else:
                result[pos] = quant * price * params[bcode][2] / 100.0
                limit -= quant
    for pos in result:
        result[pos] = round(result[pos], 2)
    return result

def calcWeightGoodsDiscount(self, weightGoods, totalQuant, params):
    '''
    Функция вычисления скидок для весовых товаров
    '''
    result = {}
    for bcode in weightGoods:
        limit = min(totalQuant[bcode] // params[bcode][0], params[bcode][1])
        if limit == 0 or totalQuant[bcode] == 0:
            continue
        k = limit / totalQuant[bcode]
        for (pos, quant, price) in weightGoods[bcode]:
            result[pos] = quant * k * price * params[bcode][2] / 100.0
    for pos in result:
        result[pos] = round(result[pos], 2)
    return result

def checkForRoundMistake(self, goods, totalQuant, params, result):
    '''
    Функция проверки значения суммы скидки
    Суммарная скидка для штрих-кода выравнивается до ожидаемой    
    '''
    for bcode in goods:
        limit = min(totalQuant[bcode] // params[bcode][0], params[bcode][1])
        if limit == 0:
            continue
        discountPure = limit * goods[bcode][0][2] * params[bcode][2] / 100.0
        discount = 0.0
        for (pos, quant, price) in goods[bcode]:
            discount += result[pos]
        if (discountPure - discount) != 0:
            result[goods[bcode][0][0]] += (discountPure - discount)
    return result

def everyNextNumberOnGoods(self, number, countLimit, percent, extops):
    '''
    Функция разварачивает чек на позиции.
    Возвращает словарь из номеров позиций в качестве ключей
    и суммы скидки в качестве значений

    number - каждый следующий номер позиции
    countLimit - количество вхождений
    percent - процент скидки
    extops - список меток товаров
    '''
    deepDiscountParams = {}
    weightGoods = {}
    pieceGoods = {}
    totalQuant = {}
    for item in self.check['goodsitems']:
        if self.checkWrapper.isPartOfKit(item['posnum']):
            continue
        
        # Для корректного прохождения тестов необходим ключ 'tmcunit'
        if 'tmcUnit' in item:
            item['tmcunit'] = item['tmcUnit']

        tmcNumber = number
        tmcCountLimit = countLimit
        tmcPercent = percent
        
        extendedOptions = ExtendedOptions(item["extendetoptions"])
        matchesExtOps = False
        
        # поиск 'discountsystem' и его параметров в extendetoptions 
        for extop in extops:
            if not extendedOptions.hasOptionLike(extop):
                continue
                
            matchesExtOps = True
            
            tempValue = extendedOptions.valueOf("ddmultiplicity")
            if tempValue:
                tmcNumber = int(tempValue)
                
            tempValue = extendedOptions.valueOf("ddmaxcount")
            if tempValue:
                tmcCountLimit = int(tempValue)
                
            tempValue = extendedOptions.valueOf("ddpercent")
            if tempValue:
                tmcPercent = int(tempValue)

            break

        if matchesExtOps:
            if item['tmcunit']['fractional']:
                if item['bcode'] not in weightGoods.keys():
                    weightGoods[item['bcode']] = list()
                    totalQuant[item['bcode']] = 0
                weightGoods[item['bcode']].append((item['posnum'], item['bquant'], item['sumb'] / item['bquant']))
            else:
                if (item['bcode']) not in pieceGoods.keys():
                    pieceGoods[item['bcode']] = list()
                    totalQuant[item['bcode']] = 0
                pieceGoods[item['bcode']].append((item['posnum'], item['bquant'], item['sumb'] / item['bquant']))
            
            totalQuant[item['bcode']] += item['bquant']
            deepDiscountParams[item['bcode']] = (tmcNumber, tmcCountLimit, tmcPercent)
    result = calcPieceGoodsDiscount(self, pieceGoods, totalQuant, deepDiscountParams)
    result.update(calcWeightGoodsDiscount(self, weightGoods, totalQuant, deepDiscountParams))

    result = checkForRoundMistake(self, weightGoods, totalQuant, deepDiscountParams, result)

    return result
