# -*- coding: utf-8 -*-
'''
Created on 25.09.2015г.

@author: kov
'''

from artixds.generalfunction.conditionhelper import __hasCatalogs


def makeDiscount(self, countGoods, resultSum, goodsList, ignorePosWithPriceEqualMin):
    """
    Скидку на весовой товар, даем только на 1кг. 
    Позиций весового товара может быть несколько, но скидка все равно на 1 кг.
    Создаем 2 списка. 
        1-й список из весовых товаров (bcode, price, [(posNum, qty)])
        2-й из не весовых (price, [(posNum, qty)])
    """

    listPositions = []
    listItemsFractional = {}

    for item in goodsList:
        if item.data['price'] <= item.data['min_price'] and ignorePosWithPriceEqualMin:
            continue

        if self.checkWrapper.isPartOfKit(item.data['posnum']):
            continue

        if item.data['tmcunit']['fractional']:
            if not item.data['bcode'] in listItemsFractional:
                listItemsFractional[item.data['bcode']] = [item.data['price'], []]
            listItemsFractional[item.data['bcode']][1].append((item.data['posnum'], item.data['bquant']))
        else:
            for pos in range(int(item.data['bquant'])):
                listPositions.append((item.data['price'], [(item.data['posnum'], item.data['bquant'])], False))

    for item in listItemsFractional:
        # Добавляем  весовой товар (sum, [(posNum, qty)])
        listPositions.append((listItemsFractional[item][0], listItemsFractional[item][1], True))

    listPositions = sorted(listPositions, key=lambda k: k[0], reverse=True)
    listPositions = listPositions[:countGoods]

    result = {}
    for position in listPositions:
        if position[2]:
            # Если позиция весовая, высчитываем пропорции скидок для всех ее позиций
            itemPrice = position[0]
            positions = position[1]
            commonSum = 0
            commonQty = 0
            for pos in positions:
                positionQty = pos[1]
                commonSum += itemPrice * positionQty
                commonQty += positionQty

            finalSumForAllItems = commonSum - (itemPrice - resultSum)
            if finalSumForAllItems < resultSum:
                finalSumForAllItems = resultSum

            remainder = finalSumForAllItems
            for pos in positions:
                posNum = pos[0]
                posQty = pos[1]
                posSum = (finalSumForAllItems * posQty / commonQty)
                posSum = round(posSum, 2)
                remainder -= posSum
                result[posNum] = (posQty * itemPrice) - posSum

            result[posNum] -= remainder

        else:
            # Иначе записываем результат
            # posNum = possum - resultSum
            posNum = position[1][0][0]

            if not posNum in result:
                result[posNum] = 0
            result[posNum] += position[0] - resultSum

    return result


def expensiveGoods(self, countGoods, conditionSum, resultSum, ignorePosWithPriceEqualMin=True):
    """
    Скидка на самый дорогой товар(ы)
        countGoods - количество товаров в чеке, на которые даем скидку;
        conditionSum - Пороговая сумма. Цена товара за 1шт/кг должна быть меньше пороговой суммы;
        resultSum - сумма, которая должна быть в результате после расчета.

    В Yuki:
        Условие - object["posnum"] in cf.expensiveGoods(2, 35, 1)
        Расчетная ставка - rf.expensiveGoods(2, 35, 1)[object["posnum"]]
    """

    if countGoods < 1:
        return

    goodsList = []
    for item in self.checkWrapper.getGoodsList():
        if item.data['price'] <= conditionSum:
            goodsList.append(item)

    return makeDiscount(self, countGoods, resultSum, goodsList, ignorePosWithPriceEqualMin)


def expensiveGoodsFromCatalogs(self, countGoods, conditionCatalogs, resultSum, ignorePosWithPriceEqualMin=True):
    """
    Скидка на самый дорогой товар(ы)
        countGoods - количество товаров в чеке, на которые даем скидку;
        conditionCatalogs - каталог, для которого действует скидка;
        resultSum - сумма, которая должна быть в результате после расчета.

    В Yuki:
        Условие - object["posnum"] in cf.expensiveGoodsFromCatalogs(2, [u"2"], 1)
        Расчетная ставка - rf.expensiveGoodsFromCatalogs(2, [u"2"], 1)[object["posnum"]]
    """

    if countGoods < 1:
        return

    goodsList = []
    for item in self.checkWrapper.getGoodsList():
        if __hasCatalogs(conditionCatalogs, item.data["tmc"]["catalogs"]):
            goodsList.append(item)

    return makeDiscount(self, countGoods, resultSum, goodsList, ignorePosWithPriceEqualMin)
