# -*- coding: utf-8 -*-
'''
Created on 31.08.2010

@author: gsu
'''
from functools import reduce

class KitEntryPosition(object):
    '''
    KitEntryPosition это элемент KitEntryItem, который есть кусок позиции чека, являющийся куском комплекта
    '''
    rosterItem = None
    quantity = 1.0
    kitEntryItem = None

    def __init__(self, rosterItem = None, quantity = 1.0, kitEntryItem=None):
        assert kitEntryItem != None
        self.rosterItem = rosterItem
        self.quantity = quantity
        self.kitEntryItem = kitEntryItem
        
    def __repr__(self):
        return "KitEntryPosition(rosterItem=%s, quantity=%f)" % (self.rosterItem, self.quantity)

class KitEntryItem(object):
    '''
    KitEntryItem это элемент KitEntry, представляющий вхождение в чек одной KitItem в виде нескольких позиций чека
    '''
    positions = []
    kitItem = None
    discountRate = None

    def __init__(self, positions = [], kitItem = None, discountRate = None):
        if positions:
            self.positions = positions
        else:
            self.positions = []
        self.kitItem = kitItem
        self.discountRate = discountRate
        
    def __iter__(self):
        for position in self.positions:
            yield position
    
    def addPosition(self, position):
        '''
        Добавить позицию к элементу
        '''
        if position:
            self.positions.append(position)
        
    def quantity(self):
        '''
        расчёт количества в элементе
        '''
        if self.positions:
            return reduce(lambda x, y: x+y, map(lambda x: x.quantity, self.positions))
        else:
            return 0.0
    
    def rosterItems(self):
        items = []
        for pos in self.positions:
            items.extend(filter(lambda x: x.kitEntryPosition.kitEntryItem == self, pos.rosterItem.rosterItem.kitIter()))
        return items
    
    def getPercentOfInitialSum(self, perc):
        return list(map(lambda item: item.sumb * float(perc) / 100.0, self.rosterItems()))

    def getPercentOfFinalSum(self, perc):
        return list(map(lambda item: item.sumb * float(perc) / 100.0, self.rosterItems()))
 
    def getAbsoluteDiscountSum(self, totalSk):
        rosterItems = self.rosterItems()
        totalSumb = sum(map(lambda x: x.sumb, rosterItems))
        leftSk = totalSk
        res = []
        for idx, x in enumerate(rosterItems):
            sk = round(totalSk * x.sumb / totalSumb, 2) if idx < len(rosterItems) - 1 else leftSk
            res.append(sk)
            leftSk -= sk
        assert(abs(leftSk) < 0.0001)
        return res
    
    def getIndexDiscountSum(self, index):
        rosterItems = self.rosterItems()
        result = []
        for item in rosterItems:
            prices = item.rosterItem.data["tmc"]["indexprices"]
            if str(index) not in prices:
                result.append(0)
            else:
                result.append( item.sumb - prices[str(index)]["price"] * item.quantity )
        return result
        
    def getDiscountSumForPrice(self, price):
        rosterItems = self.rosterItems()
        totalSumb = sum(map(lambda x: x.sumb, rosterItems))
        return self.getAbsoluteDiscountSum(totalSumb - price)

    def __repr__(self):
        return "KitEntryItem(positions=%s)" % self.positions

class KitEntry(object):
    '''
    KitEntry is the real KitObject which present in Check
    '''
    items = []
    
    def __init__(self, items = []):
        if items:
            self.items = items
        else:
            self.items = []

    def __iter__(self):
        for item in self.items:
            yield item
            
    def addItem(self, item):
        if item:
            self.items.append(item)
    
    def __repr__(self):
        return "KitEntry(items=%s)" % self.items

    def positions(self):
        for item in self.items:
            for pos in item:
                yield pos
