# -*- coding: utf-8 -*-
'''
Created on 25.06.2010

@author: adm
'''

import logging
from enum import Enum
from artixds.discountrate import CalculateDiscountRate, DiscountRateType

log = logging.getLogger("Kit")


class KitType(Enum):
    CODES = 'codes'
    CATALOGS = 'catalogs'


class DiscountObject(object):
    """
    Object
    """
    def getPercentOfInitialSum(self, perc):
        return (self.getInitialSum() * perc) / 100.0

    def getPercentOfFinalSum(self, perc):
        return (self.getFinalSum() * perc) / 100.0

    def getAbsoluteDiscountSum(self, totalSk):
        return totalSk

    def getDiscountSumForPrice(self, price):
        return (self.getPrice() - price) * self.getQuantity()


class CheckObject(DiscountObject):
    """ Check object """
    def __init__(self, data):
        """ data - dictionary with check content """
        self.data = data

    def __repr__(self):
        return "<CheckObject()>"

    def getInitialSum(self):
        return self.data['sum2']

    def getFinalSum(self):
        return self.data['sumb']

    def getQuantity(self):
        return 1.0

    def getPrice(self):
        return self.data['sumb']


class PositionObjectModifiers(object):
    def __init__(self, posnum, partOfKit, cquant, bquant, sumi, sumn, sumb, sume, packingMode):
        self.posnum = posnum
        self.cquant = cquant
        self.bquant = bquant
        self.partOfKit = partOfKit
        self.sumb = sumb
        self.sumn = sumn
        self.sume = sume
        self.sumi = sumi
        self.packingMode = packingMode
        self.active = True
        self.kitEntryPosition = None
        self.impact = None


class PositionObjectData(object):
    def __init__(self, data, modifiers=None):
        self.data = data
        self.modifiers = modifiers
        self.discountApplied = False
        self.hasDiscount = False
        self.hasBonus = False
        self.hasImpact = False

    def __contains__(self, key):
        return key in self.data or hasattr(self, key) or hasattr(self.modifiers, key)

    def __getitem__(self, key):
        if self.modifiers and hasattr(self.modifiers, key):
            return getattr(self.modifiers, key, None)
        if key in self.data:
            return self.data[key]
        return getattr(self, key, None)
        #  return getattr(self.modifiers, key, None) if self.modifiers and hasattr(self.modifiers, key) else self.data[key]


class PositionObject(DiscountObject):
    """
    Position object
    """
    def __init__(self, data, modifiers=None):
        """
        data - dictionary with position content
        """
        self.data = PositionObjectData(data, modifiers)

    def __repr__(self):
        code = None
        if "code" in self.data:
            code = self.data["code"]
        bcode = None
        if "bcode" in self.data:
            bcode = self.data["bcode"]
        return "<PositionObject('%s','%s')>" % (repr(code), repr(bcode))

    def markAsDiscountApplied(self):
        # Старая функция. Оставлена для совместимости.
        self.data.discountApplied = True

    def markAsDiscount(self):
        self.data.hasDiscount = True

    def markAsBonus(self):
        self.data.hasBonus = True

    def markAsImpact(self):
        self.data.hasImpact = True

    def getInitialSum(self):
        return self.data['sumi']

    def getFinalSum(self):
        return self.data['sumb']

    def getQuantity(self):
        return self.data['bquant']

    def getIndexDiscountSum(self, index):
        prices = self.data["tmc"]["indexprices"]
        if str(index) not in prices:
            return 0
        if "tags" in self.data and "social" in self.data["tags"].split(";"):
            return 0
        return self.getFinalSum() - prices[str(index)]["price"] * self.data["bquant"]

    def getPrice(self):
        return self.data['price']


class KitItem(object):
    """
    KitItem is the one item of Kit. It consists of list of codes and quantity
    """
    codes = []
    quantity = 1.0

    '''
    ChaykaA
    Added name
    Added inventGroups[]
    Added extendedOptions[]
    '''
    inventGroups = []
    extendedOptions = []
    conditions = []

    def __init__(self, codes=[], inventGroups=[], extendedOptions=[],  quantity=1, name=None):
        self.codes = codes if codes else []
        self.inventGroups = inventGroups if inventGroups else []
        self.extendedOptions = extendedOptions if extendedOptions else []
        self.name = name
        self.quantity = quantity

    def __repr__(self):
        return "KitItem(codes=%s,quantity=%f)" % (self.codes, self.quantity)


class Kit(DiscountObject):
    """
    Kit is the representation of Kit Object in Discount
    """
    items = []
    id = None
    name = None
    kittype = KitType.CODES.value
    maxtimes = None
    byTmc = None
    cartesian = None

    def __init__(self, items=None, id=None, name=None, kittype=KitType.CODES.value):
        self.items = [] if items is None else items
        self.id = id
        self.name = name
        self.kittype = kittype
        self.data = self.__dict__

    def maxOccurances(self, checkWrapper):
        """
        Получение максимального количества вхождений набора в чек
        checkWrapper - чек
        maxtimes = None - ограничение отсутствует
        """
        if self.maxtimes:
            rate = CalculateDiscountRate(value=str(self.maxtimes), type=DiscountRateType.SUM.value)
            rate.checkWrapper = checkWrapper
            return rate.getAbsoluteValue(check=checkWrapper.checkDict, obj=self)
        return 9999999

    def __iter__(self):
        for item in self.items:
            yield item

    def __len__(self):
        return len(self.items)

    def __repr__(self):
        return "KitObject(items=%s)" % self.items


class LegacyObject(DiscountObject):
    """
    Legacy object
    """
    pass


class SimpleObject(DiscountObject):
    """
    Simple object
    """
    pass
