# -*- coding: utf-8 -*-

import sys
import ujson

if sys.version_info.major == 3:
    import configparser
else:
    import ConfigParser as configparser


class JsonConfigProvider(object):
    """
    {
      'config': {}
      'metadata': {}
    }
    """
    def __init__(self, jsonStr='{}'):
        data = ujson.loads(jsonStr)
        self.config = data.get('config', {})
        self.okSet = self.config.get('metadata', {}).get('okSet', ["ok", "yes", "enable", "1", "true", "allow", "on", "y", "да"])

    def getBool(self, section, option, defaultValue):
        if JsonConfigProvider.__optName(section, option) not in self.config:
            return defaultValue
        return str(self.config.get(JsonConfigProvider.__optName(section, option))).lower() in self.okSet

    def getInt(self, section, option, defaultValue):
        return int(self.config.get(JsonConfigProvider.__optName(section, option), defaultValue))

    def getString(self, section, option, defaultValue):
        return str(self.config.get(JsonConfigProvider.__optName(section, option), defaultValue))

    @staticmethod
    def __optName(section, option):
        return (section + ':' + option).lower()


class IniConfigProvider(object):
    okSet = ["ok", "yes", "enable", "1", "true", "allow", "on", "y", "да"]

    def __init__(self, fileName):
        self.config = configparser.RawConfigParser()
        self.config.read(fileName)

    def getBool(self, section, option, defaultValue):
        if self.config.has_section(section) and self.config.has_option(section, option):
            return IniConfigProvider.simplify(self.config.get(section, option)) in IniConfigProvider.okSet
        return defaultValue

    def getInt(self, section, option, defaultValue):
        if self.config.has_section(section) and self.config.has_option(section, option):
            return int(IniConfigProvider.simplify(self.config.get(section, option)))
        return defaultValue

    def getString(self, section, option, defaultValue):
        if self.config.has_section(section) and self.config.has_option(section, option):
            return IniConfigProvider.simplify(self.config.get(section, option))
        return defaultValue

    @staticmethod
    def simplify(value):
        return value.replace('"', '').replace('\'', '').replace(';', '')


class Config(object):
    """ Настройки дисконта
    Может брать настройки из объекта ArtixConfig или ini файла
    Ini файл может потом пригодится для тестов
    loadFromArtixConfig - вызывается из кассы
    """
    _instance = None
    data = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(Config, cls).__new__(cls, *args, **kwargs)
            cls.data = JsonConfigProvider()
        return cls._instance

    def reset(self):
        Config.data = JsonConfigProvider()

    def fromJsonConfig(self, jsonStr):
        Config.data = JsonConfigProvider(jsonStr)

    def loadFromIni(self, fileName):
        Config.data = IniConfigProvider(fileName)

    def getBool(self, section, option, defaultValue):
        return Config.data.getBool(section, option, defaultValue)

    def getInt(self, section, option, defaultValue):
        return Config.data.getInt(section, option, defaultValue)

    def getString(self, section, option, defaultValue):
        return Config.data.getString(section, option, defaultValue)
