# -*- coding: utf-8 -*-

import copy
from datetime import datetime
import re
import sys


class QTime(object):
    def __init__(self, time):
        self.time = time
            
    def hour(self):
        return self.time.hour
    
    def minute(self):
        return self.time.minute
    
    def second(self):
        return self.time.second
    

class QDate(object):
    def __init__(self, value):
        self.date = datetime.strptime(value, "%Y-%m-%d").date() if value else None

    def year(self):
        return self.date.year

    def month(self):
        return self.date.month

    def day(self):
        return self.date.day
    
    def dayOfWeek(self):
        return self.date.isoweekday() 

    def isValid(self):
        return self.date is not None


class QDateTime(object):
    def __init__(self, value):
        try:
            if value:
                withoutMsecs = re.sub(r'([\d\-T:]+)(\.\d+)', r'\1', value)
                self.datetime = datetime.strptime(withoutMsecs, "%Y-%m-%dT%H:%M:%S")
            else:
                self.datetime = datetime.fromtimestamp(0)
        except:
            self.datetime = value

    def __cmp__(self, other):
        if self.datetime < other.datetime:
            return -1
        elif self.datetime > other.datetime:
            return 1
        return 0

    def isValid(self):
        return self.datetime != datetime.fromtimestamp(0)

    def toString(self, format="yyyy-MM-ddThh:mm:ss"):
        return self.__fmtDateTime(format, self.datetime, self.datetime)

    def date(self):
        return QDate(self.datetime.strftime('%Y-%m-%d'))

    def time(self):
        return QTime(self.datetime)

    def __strftime(self, dt, format):
        return dt.strftime(format).decode('utf-8') if sys.version_info.major == 2 else dt.strftime(format)

    def __getFmtString(self, f, dt=None, dd=None, am_pm=False):
        if not f:
            return ''

        buf = f
        removed = 0

        if dt:
            if f.startswith('hh') or f.startswith('HH'):
                hour12 = f[0] == 'h' and am_pm
                if hour12 and dt.hour > 12:
                    buf = '%02d' % (dt.hour - 12)
                elif hour12 and dt.hour == 0:
                    buf = '12'
                else:
                    buf = '%02d' % dt.hour
                removed = 2
            elif f[0] == 'h' or f[0] == 'H':
                hour12 = f[0] == 'h' and am_pm
                if hour12 and dt.hour > 12:
                    buf = str(dt.hour - 12)
                elif hour12 and dt.hour == 0:
                    buf = "12"
                else:
                    buf = str(dt.hour)
                removed = 1
            elif f.startswith("mm"):
                buf = '%02d' % dt.minute
                removed = 2
            elif f[0] == 'm':
                buf = str(dt.minute)
                removed = 1
            elif f.startswith("ss"):
                buf = '%02d' % dt.second
                removed = 2
            elif f[0] == 's':
                buf = str(dt.second)
            elif f.startswith("zzz"):
                buf = '%03d' % (dt.microsecond // 1000)
                removed = 3
            elif f[0] == 'z':
                buf = str(dt.microsecond // 1000)
                removed = 1
            elif f[0].upper() == 'A':
                upper = f[0] == 'A'
                buf = "am" if dt.hour < 12 else "pm"
                if upper:
                    buf = buf.upper()
                if len(f) > 1 and f[1].upper() == 'P' and f[0].isupper() == f[1].isupper():
                    removed = 2
                else:
                    removed = 1

        if dd:
            if f.startswith("dddd"):
                buf = self.__strftime(dd, '%A')
                removed = 4
            elif f.startswith("ddd"):
                buf = self.__strftime(dd, '%a')
                removed = 3
            elif f.startswith("dd"):
                buf = '%02d' % dd.day
                removed = 2
            elif f[0] == 'd':
                buf = str(dd.day)
                removed = 1
            elif f.startswith("MMMM"):
                buf = self.__strftime(dd, '%B')
                removed = 4
            elif f.startswith("MMM"):
                buf = self.__strftime(dd, '%b')
                removed = 3
            elif f.startswith("MM"):
                buf = '%02d' % dd.month
                removed = 2
            elif f[0] == 'M':
                buf = str(dd.month)
                removed = 1
            elif f.startswith("yyyy"):
                year = dd.year
                buf = '%04d' % abs(year)
                if year > 0:
                    removed = 4
                else:
                    buf = buf + '-'
                    removed = 5

            elif f.startswith("yy"):
                buf = '%02d' % dd.year
                removed = 2

        if removed == 0 or removed >= len(f):
            return buf

        return buf + self.__getFmtString(f[removed:], dt, dd, am_pm)

    def __hasUnquotedAP(self, f):
        quote = '\''
        inquote = False
        max = len(f)
        for ch in f:
            if ch == quote:
                inquote = not inquote
            elif not inquote and ch.upper() == 'A':
                return True
        return False

    def __fmtDateTime(self, f, dt, dd):
        quote = '\''
        if not f:
            return ''

        ap = self.__hasUnquotedAP(f)

        buf = ''
        frm = ''
        status = '0'

        for i in range(0, len(f)):
            if f[i] == quote:
                if status == quote:
                    if i > 0 and f[i - 1] == quote:
                        buf += '\''
                        status = '0'
                else:
                    if frm:
                        buf += self.__getFmtString(frm, dt, dd, ap)
                        frm = ''
                    status = quote
            elif status == quote:
                buf += f[i]
            elif f[i] == status:
                if ap and ((f[i] == 'P') or (f[i] == 'p')):
                    status = '0'
                frm += f[i]
            else:
                buf += self.__getFmtString(frm, dt, dd, ap)
                frm = ''
                if (f[i] == 'h') or (f[i] == 'm') or (f[i] == 'H') or (f[i] == 's') or (f[i] == 'z'):
                    status = f[i]
                    frm += f[i]
                elif (f[i] == 'd') or (f[i] == 'M') or (f[i] == 'y'):
                    status = f[i]
                    frm += f[i]
                elif ap and (f[i] == 'A'):
                    status = 'P'
                    frm += f[i]
                elif ap and (f[i] == 'a'):
                    status = 'p'
                    frm += f[i]
                else:
                    buf += f[i]
                    status = '0'

        buf += self.__getFmtString(frm, dt, dd, ap)
    
        return buf
        

class Dict(object):
    def __init__(self, value):
        self.dictionary = Dict.keysToLower(value)

    def get(self, key, default=None):
        return self.dictionary.get(key.lower(), default)

    @staticmethod
    def keysToLower(d):
        if isinstance(d, list):
            mass = []
            for value in d:
                mass.append(Dict.keysToLower(value))
            return mass
        elif isinstance(d, dict):
            t = {}
            for key in d.keys():
                k, v = Dict.convitem(key.lower(), d[key])
                t[k] = Dict.keysToLower(v)
            return t
        else:
            return d

    @staticmethod
    def convitem(k, v):
        if k.startswith('datetime') or k in ['multiplicatorbeg', 'multiplicatorend', 'item_datetime']:
            return k, QDateTime(v)
        elif k in ['birthday', 'specialdate1', 'specialdate2', 'specialdate3']:
            return k, QDate(v)
        return k, v


class BaseArtixObject(object):
    def __init__(self, value):
        self.data = Dict(value)

    def __getitem__(self, key):
        return self.data.dictionary.get(key.lower())

    def __getattr__(self, item):
        return self.data.dictionary.get(item.lower())

    def __deepcopy__(self, memodict={}):
        """ Нужен из-за переопределённого метода __getattr__ """
        return copy.deepcopy(self.data)

    def toDict(self):
        return self.data.dictionary


class GoodsItem(BaseArtixObject):
    def __init__(self, value):
        super(GoodsItem, self).__init__(value)
        self.tmc = Tmc(value.get('tmc', {}))

    def getTmcObject(self):
        return self.tmc


class CatalogsList(BaseArtixObject):
    def __init__(self, value):
        super(CatalogsList, self).__init__(value)


class TmcIndexPrice(BaseArtixObject):
    def __init__(self, value):
        super(TmcIndexPrice, self).__init__(value)


class Tmc(BaseArtixObject):
    def __init__(self, value):
        super(Tmc, self).__init__(value)
        self.catalogs = [CatalogsList(i) for i in value.get('catalogs', [])]

    def getCatalogsList(self):
        return self.catalogs
    
    def getIndexPriceObject(self, index):
        data = self.data.get('indexPrices', {}).get(str(index), None)
        return TmcIndexPrice(data) if data is not None else None


class MoneyItem(BaseArtixObject):
    def __init__(self, value):
        super(MoneyItem, self).__init__(value)


class DiscountItem(BaseArtixObject):
    def __init__(self, value):
        super(DiscountItem, self).__init__(value)


class Card(BaseArtixObject):
    def __init__(self, value):
        super(Card, self).__init__(value)


class CardItem(BaseArtixObject):
    def __init__(self, value):
        super(CardItem, self).__init__(value)
        self.card = Card(value.get('card', {}))

    def getCardObject(self):
        return self.card

    def getDump(self):
        return self.card


class BonusItem(BaseArtixObject):
    def __init__(self, value):
        super(BonusItem, self).__init__(value)


class ButtonItem(BaseArtixObject):
    def __init__(self, value):
        super(ButtonItem, self).__init__(value)


class DepartmentItem(BaseArtixObject):
    def __init__(self, value):
        super(DepartmentItem, self).__init__(value)


class BasePaymentItem(BaseArtixObject):
    def __init__(self, value):
        super(BasePaymentItem, self).__init__(value)


class Document(BaseArtixObject):
    """ Аналог BasicDocument из кассы """
    def __init__(self, value):
        super(Document, self).__init__(value)
        self.priceType = None
        self.quantity = sum([i['bquant'] for i in value.get('goodsItems', [])])
        self.goodsItems = [GoodsItem(i) for i in value.get('goodsItems', [])]
        self.stornoGoodsItems = [GoodsItem(i) for i in value.get('stornoItems', [])]
        self.moneyItems = [MoneyItem(i) for i in value.get('moneyItems', [])]
        self.discountItems = [DiscountItem(i) for i in value.get('discounts', [])]
        self.cardItems = [CardItem(i) for i in value.get('cardRecords', [])]
        self.bonusItems = [BonusItem(i) for i in value.get('bonusRecords', [])]

    def getGoodsList(self):
        return self.goodsItems

    def getMoneyList(self):
        return self.moneyItems

    def getDiscountList(self):
        return self.discountItems

    def getCardsList(self):
        return self.cardItems

    def getBonusList(self):
        return self.bonusItems

    def getButtonObject(self):
        return ButtonItem(self.data.get('buttonItem'))

    def getDepartmentObject(self):
        return ButtonItem(self.data.get('department'))

    def getBasePaymentObject(self):
        return BasePaymentItem(self.data.get('basepaymenttype'))

    def getNextPosNum(self):
        nums = [0]
        nums.extend([i["posnum"] for i in self.goodsItems if i["posnum"] is not None])
        nums.extend([i["posnum"] for i in self.stornoGoodsItems if i["posnum"] is not None])
        return max(nums) + 1

    def getDateTimeBeg(self):
        return self.data.get('dateTimeBeg')

    def getDynamicProperties(self):
        return self.data.get('dynamicProperties', {})
