# -*- coding: utf-8 -*-

from sqlalchemy import Column, Integer, String, Numeric, Date
from legacydiscount.connection.connection import Base, Session
from config import Config

session = Session()


class DiscCard(Base):
    """ Дисконтная карта """

    prefix = Config().getString('Discounts', 'cardPrefixStartsWith', '')

    __tablename__ = 'disccard'

    cardnumber = Column(String(128), primary_key=True)
    name = Column(String(100))
    rtext = Column(String(30))
    cardsum = Column(Numeric(precision=17, scale=2))
    discasheme = Column(Integer)
    denycard = Column(Integer)
    notaccumulate = Column(Integer)
    positionmode = Column(Integer)
    ispaycard = Column(Integer)
    allpositions = Column(Integer)
    msgsheme = Column(Integer)
    rcount = Column(Numeric(precision=20, scale=0))
    acdiscscheme = Column(Integer)
    cardmode = Column(Integer)
    bonussum = Column(Numeric(precision=17, scale=2))
    bonuscode = Column(Integer)
    groupcode = Column(Integer)
    birthday = Column(Date)
    acdiscasheme = Column(Integer)

    @classmethod
    def select(cls, cardnumber):
        res = session.query(cls).filter_by(cardnumber=cardnumber).first()
        if not res and cls.prefix:
            query = session.query(cls).filter(cls.cardnumber.like('%s%%' % cls.prefix))
            query = query.filter_by(denycard=0).filter_by(cardmode=0)
            res = query.all()
            for res in query:
                if (cls.prefix + cardnumber).startswith(res.cardnumber):
                    return res
            res = None
        return res

    def __str__(self):
        return u'Discount Card (number: "%s", name: "%s", discascheme: %s, positionmode: %s, allpositions: %s)' % (self.cardnumber, self.name, self.discasheme, self.positionmode, self.allpositions)

    def __repr__(self):
        return str(self)

    def __eq__(self, other):
        return self.cardnumber == other.cardnumber

    def __hash__(self):
        return hash(self.cardnumber)
