# -*- coding: utf-8 -*-
'''
Created on 10.02.2011

@author: gsu
'''

from decimal import Decimal
from enum import Enum
import logging
from legacydiscount.compare import coalesce
from legacydiscount.bunch import Bunch

logger = logging.getLogger('su.artix.dm')

class RateType(Enum):
    PERCENT = 'percent'
    SUM = 'sum'
    INDEX = 'index'

class DiscountRate(object):
    value = Decimal(0)
    times = None


    def impactsOnObjects(self, objects):
        return [ self.appliedValue(object) for object in objects ]

class ZeroDiscountRate(DiscountRate):
    _Zero = Decimal('0.0')
    type = RateType.PERCENT.value
    discval = property(lambda self: self._Zero)
    scheme = Bunch(code=0, name=u'Часть комплекта')
    disccode = 0
    name = u'Часть комплекта'

    def appliedValue(self, object):
        return self._Zero

    def appliedToObjects(self, objects):
        return [ self.appliedValue(object) for object in objects ]

class PercentDiscountRate(DiscountRate):
    
    _100 = Decimal('100.0')

    type = RateType.PERCENT.value
    
    def __init__(self, value):
        self.value = value
        self.type = RateType.PERCENT.value

    def appliedValue(self, object):
        return self.value * object.initialSum / self._100

    def appliedToObjects(self, objects):
        return [ self.appliedValue(object) for object in objects ]

class SumDiscountRate(DiscountRate):
    
    type = RateType.SUM.value
    
    def __init__(self, value):
        self.value = value
        self.ratevalue = {}
        self.type = RateType.SUM.value
        
    def appliedValue(self, object):  
        if(self.ratevalue):     
            return self.ratevalue[object.posnum]
        else:
            return self.value
    def setTimes(self,times):
        self.times = times

    def appliedToObjects(self, objects):
        
        val = 0.0  
           
        total = float(sum([ object.initialSum for object in objects]))       
        remain = totalrate = float(self.value * self.times)
        for i, object in enumerate(objects): 
            if (object.initialSum < 0.001):
                self.ratevalue[object.posnum] = 0     
                continue         
            if i < len(objects) - 1:                
                val = round(float(object.initialSum) / total * totalrate,2)     
                if (remain - val < 0.001 ):                   
                    val = remain - val                 
            else:
                val = remain              
            self.ratevalue[object.posnum] = val            
            remain -=val   

class IndexDiscountRate(object):
    
    type = RateType.INDEX.value
    
    def __init__(self, value):
        self.value = value
        self.type = RateType.INDEX.value
    
    def appliedValue(self, object):
        return object.initialSum - object.indexSum(self.value)

class DiscountRateMixin:

    _rate = None
    _Zero = Decimal(0)

    
    def _discountRate(self):
        
        if not self._rate:
            val = coalesce(self.discval, self._Zero) if self.disctype == 1 else -coalesce(self.discval, self._Zero)    
            cls = PercentDiscountRate if self.discmode == 1 else SumDiscountRate            
            if (not self.discval or self.discval == self._Zero) and hasattr(self,'pricecode') and self.pricecode and self.pricecode != 0:
                cls = IndexDiscountRate
                val = self.pricecode
            self._rate = cls(val)
        return self._rate  
    
    def appliedValue(self, object):
        return self._discountRate().appliedValue(object)

    def appliedToObjects(self, objects):
        return self._discountRate().appliedToObjects(objects)
   
    def impactsOnObjects(self, objects):
        return self._discountRate().impactsOnObjects(objects)
 
    def setTimes(self, times): 
        if hasattr(self._discountRate(),'setTimes'): 
            self._discountRate().setTimes(times)

    type = property(lambda self: self._discountRate().type)
    value = property(lambda self: self._discountRate().value)
