# -*- coding: utf-8 -*-
'''
Created on 04.02.2011

@author: gsu
'''

from sqlalchemy import Column, Integer, SmallInteger, String, Numeric, Time, Date, ForeignKey, MetaData
from sqlalchemy.orm import relation, backref, outerjoin
from sqlalchemy.sql import func, and_, expression
from legacydiscount.connection.connection import Base, Session
from legacydiscount.uniq import uniq
from decimal import Decimal
from legacydiscount.compare import between, coalesce
from legacydiscount.discountrate import DiscountRateMixin
import logging

session = Session()
metadata = MetaData(bind = session.get_bind(None))
#goodsTable = Table('tmpshortgoodslist', metadata, autoload = True)

logger = logging.getLogger('su.artix.dm')

class Campaign(Base):
    
    __tablename__ = 'campaign'
    
    campaignid = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String(100))
    
    code = property(lambda self: self.campaignid)
    
#    kits = relation('Kit', order_by='Kit.kitid', backref='campaign')

    @classmethod
    def select(cls):
        return session.query(cls).all()

    def __str__(self):
        return u'Campaign (id: %d, name: "%s")' % (self.campaignid, self.name)
    
    def __repr__(self):
        return str(self)

    def __eq__(self, other):
        return self.campaignid == other.campaignid

    def __lt__(self, other):
        return self.campaignid < other.campaignid

    def __hash__(self):
        return hash(self.campaignid)

    def isActiveForDateTime(self, datetime):
        for tp in self.timeperiods:
            if tp.isActiveForDateTime(datetime):
                return True
        return False

    def __cmp__(self, other):
        return self.campaignid - other.campaignid

    @classmethod
    def selectActive(cls, datetime):
        all = session.query(Campaign).select_from(outerjoin(cls, TimePeriod)).filter(and_(
                                              func.coalesce(TimePeriod.date_start, datetime.date()) <= datetime.date(),
                                              func.coalesce(TimePeriod.date_end, datetime.date()) >= datetime.date(),
                                              func.coalesce(TimePeriod.time_start, datetime.time()) <= datetime.time(),
                                              func.coalesce(TimePeriod.time_end, datetime.time()) >= datetime.time(),
                                              func.coalesce(TimePeriod.day_start, datetime.weekday() + 1) <= datetime.weekday() + 1,
                                              func.coalesce(TimePeriod.day_end, datetime.weekday() + 1) >= datetime.weekday() + 1
                                              )).all()
        return sorted(uniq(all))

class TimePeriod(Base):
    
    __tablename__ = 'timeperiod'
    
    campaignid = Column(Integer, ForeignKey(Campaign.campaignid))
    date_end = Column(Date)
    date_start = Column(Date)
    day_end = Column(Integer)
    day_start = Column(Integer)
    timeperiodid = Column(Integer, primary_key=True, autoincrement=True)
    time_end = Column(Time(timezone=False))
    time_start = Column(Time(timezone=False))
    
    campaign = relation(Campaign, backref=backref('timeperiods', order_by='TimePeriod.timeperiodid'))
    
    def _days(self):
        return range(coalesce(self.day_start, 1), coalesce(self.day_end, 7) + 1)
            
    def isActiveForDateTime(self, datetime):
        if between(datetime.date(), self.date_start, self.date_end) and between(datetime.time(), self.time_start, self.time_end):
            if datetime.weekday() + 1 in self._days():
                return True

    @classmethod
    def selectActive(cls, datetime):
        return session.query(cls).filter(and_(func.coalesce(cls.date_start, datetime.date()) <= datetime.date(),
                                              func.coalesce(cls.date_end, datetime.date()) >= datetime.date(),
                                              func.coalesce(cls.time_start, datetime.time()) <= datetime.time(),
                                              func.coalesce(cls.time_end, datetime.time()) >= datetime.time(),
                                              func.coalesce(cls.day_start, datetime.weekday() + 1) >= datetime.weekday() + 1,
                                              func.coalesce(cls.day_end, datetime.weekday() + 1) <= datetime.weekday() + 1
                                              )).all()

class Kit(Base):
    
    __tablename__ = 'kit'
    
    campaignid = Column(Integer, ForeignKey(Campaign.campaignid))
    kitid = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String(100))
    kittype = Column(SmallInteger)
    maxtimes= Column(Integer) 
    campaign = relation(Campaign, backref=backref('kits', order_by='Kit.kitid'))
    
    def __str__(self):
        return u'Kit (campaign id: %d, id: %d, name: "%s")' % (self.campaignid, self.kitid, self.name)

    def __repr__(self):
        return str(self)

    def __eq__(self, other):
        return self.kitid == other.kitid

    def __lt__(self, other):
        return self.kitid < other.kitid

    def __hash__(self):
        return hash(self.kitid)

    def __cmp__(self, other):
        return self.campaignid - other.campaignid if self.campaignid != other.campaignid else self.kitid - other.kitid

    @classmethod
    def selectContains(cls, goodsListShort, campaigns = None, goodsTable = None, selectByCode = False):
        '''
            Функция поиска наборов, которые могут быть выделены в чеке
            при selectByCode = True выбираются наборы сформированные каталогами, иначе - кодами товаров
            goodsListShort - свертка чека вида "код: количество" 
            campign - активные на данный момент акции
            goodsTable - временная таблица
        '''
        session.expire_all()
        session.execute(goodsTable.delete())
        items = [ {"code" : k , "quantity" : v} for k, v in goodsListShort.items() ]
        if len(items):
            session.execute(goodsTable.insert(), items)
        kitSubquery = session.query(KitGoods.kitid).join((goodsTable, KitGoods.goodscode==goodsTable.c.code)).group_by(KitGoods.kitid).subquery()

       
        kitDetailSubquery = session.query(KitGoods.kitid, KitGoods.goodscode, expression.literal_column('1').label('quant2'), \
                expression.case([(func.sum(KitGoods.quantity) <= goodsTable.c.quantity, expression.literal_column('1'))], \
                                else_ = expression.literal_column('0')).label('quant1')) \
                .outerjoin((goodsTable, KitGoods.goodscode==goodsTable.c.code)).join((kitSubquery, KitGoods.kitid==kitSubquery.c.kitid)) \
                .group_by(KitGoods.kitid, KitGoods.goodscode, goodsTable.c.quantity).subquery()
      
        kitIdSubquery = session.query(kitDetailSubquery.c.kitid).group_by(kitDetailSubquery.c.kitid).having(func.sum(kitDetailSubquery.c.quant1) \
                .label('s1') == func.sum(kitDetailSubquery.c.quant2).label('s2')).subquery()
        
        ktype = 0 if (selectByCode) else 1
        if not campaigns:
            all = []
        else:
            campaignids = uniq([ c.campaignid for c in campaigns])
            all = session.query(Kit).join((kitIdSubquery, Kit.kitid==kitIdSubquery.c.kitid))\
                .filter(and_(Kit.campaignid.in_(campaignids), func.coalesce(Kit.kittype,0) == ktype)).all()
                
        logger.debug('%d kitgoods matched', len(all))
        return all
    
    @classmethod
    def selectActiveForCodes(cls, datetime, goodsListShort, goodsTable):
        return sorted(Kit.selectContains(goodsListShort, Campaign.selectActive(datetime), goodsTable, True))

    @classmethod
    def selectActiveForCatalogs(cls, datetime, catalogsListShort, goodsTable):
        return sorted(Kit.selectContains(catalogsListShort, Campaign.selectActive(datetime), goodsTable, False))


    def goodscodes(self):
        return uniq([ kg.goodscode for kg in self.kitgoods ])

    def goodscq(self):
        resmap = {}
        for c, q in [ (item.goodscode, item.quantity) for item in self.kitgoods ]:
            resmap[c] = resmap.get(c, Decimal(0)) + q
        return resmap

class KitGoods(Base):
    
    __tablename__ = 'kitgoods'
    
    goodscode = Column(String(100))
    kitgoodsid = Column(Integer, primary_key=True, autoincrement=True)
    kitid = Column(Integer, ForeignKey(Kit.kitid))
    quantity = Column(Numeric(precision=14, scale=3))

    kit = relation(Kit, backref=backref('kitgoods', order_by='KitGoods.kitgoodsid'))

    def __str__(self):
        return u'Kit Goods (code: "%s", id: %d, kit id: %d, quantity: %.2f)' % (self.goodscode, self.kitgoodsid, self.kitid, self.quantity)
    
    def __repr__(self):
        return str(self)

    def __hash__(self):
        return hash(self.kitgoodsid)

    @classmethod
    def selectContainsCodes(cls, codes, campaigns = None):
        if not campaigns:
            all = session.query(cls).filter(cls.goodscode.in_(codes)).all()
            logger.debug('%d kitgoods matched', len(all))
            return all
        else:
            campaignids = uniq([ c.campaignid for c in campaigns])
            all = session.query(cls).join(Kit).filter(and_(cls.goodscode.in_(codes), Kit.campaignid.in_(campaignids))).all()
            logger.debug('%d kitgoods matched', len(all))
            return all

    def __eq__(self, other):
        return self.kitgoodsid == other.kitgoodsid

class KitDisc(Base, DiscountRateMixin):
    
    __tablename__ = 'kitdisc'
    
    disckind = Column(Integer)
    disctype = Column(Integer)
    discval = Column(Numeric(precision=14, scale=3))
    kitdiscid = Column(Integer, autoincrement=True, primary_key=True)
    kitgoodsid = Column(Integer, ForeignKey(KitGoods.kitgoodsid))
    impactmode = Column(Integer)
    
    discmode = property(lambda self: self.disckind)

    kitgoods = relation(KitGoods, backref=backref('kitdiscs', order_by='KitDisc.kitdiscid'))
    campaign = property(lambda self: self.kitgoods.kit.campaign)
    
    scheme = property(lambda self: self.campaign)
    disccode = property(lambda self: self.kitdiscid)
    name = property(lambda self: self.kitgoods.kit.name)

    def __str__(self):
        return u'Kit Disc (id: %d, kind: %d, type: %d, val: %.2f, kit goods id: %d, impactmode: %d)' % (self.kitdiscid, self.disckind, self.disctype, self.discval, self.kitgoodsid, self.impactmode)

    def __repr__(self):
        return str(self)

    def __eq__(self, other):
        return self.kitdiscid == other.kitdiscid

    def __hash__(self):
        return hash(self.kitdiscid)
