# -*- coding: utf-8 -*-

from legacydiscount.discountmanager import DiscountManager, KitManager, DiscountCollection, Discount, DiscountMode
from legacydiscount.checktotals import CheckTotals, DiscountObjectForCheck
from os import path
import jsonpickle
import logging.config
from legacydiscount.log_decorator import log_after
from legacydiscount.bunch import Bunch
from legacydiscount.discountrate import PercentDiscountRate
from document import Document
import ujson

try:
    logging.config.fileConfig('/linuxcash/cash/discountsystems/log.ini')
except:
    logging.basicConfig()

logger = logging.getLogger('su.artix.dm')

km = KitManager()


def refreshSystems():
    pass


@log_after
def encodeResult(d, b):
    return jsonpickle.encode([ [ q.toDict() for q in d ], b ])


@log_after
def calculateImpact(check):
    totals = CheckTotals(check)
    dm = DiscountManager(totals)
    km.extractKits(totals)
    b, d = km.discounts(totals)
    d.extend(dm.discount(check))
    for card in check.getCardsList():
        d.extend(dm.discountByCard(check, card))
    return b, d


@log_after
def calculateDiscountImpact():
    logger.debug('entering calculateDiscountImpact')
    document = Document(ujson.loads(jsonDocument))
    totals = CheckTotals(document)
    dm = DiscountManager(totals)
    km.extractKits(totals)
    b, d = km.discounts(totals)
    d.extend(dm.discount(document))
    logger.debug('there are %d cards in check found' % len(document.getCardsList()))
    for card in document.getCardsList():
        logger.info("scanning for card '%s'" % card.getCardObject().number)
        d.extend(dm.discountByCard(document, card))
    res = encodeResult(d, b)
    logger.debug('result is %s' % res)
    return res


def calculateBonusEarningsImpact():
    return None


def calculateBonusLimitImpact():
    document = Document(ujson.loads(jsonDocument))
    totals = CheckTotals(document)
    dobj = DiscountObjectForCheck(document)
    rate = PercentDiscountRate(100)
    rate.scheme = Bunch(code=0, name=u'')
    rate.disccode = 0
    rate.name = ''
    return encodeResult(DiscountCollection([Discount(dobj, dobj, dobj, rate, None, DiscountMode.AUTOMATIC.value)]), [])


if __name__ == '__main__':
    ck = u'check.json'
    if path.isfile(ck):
        logger.info('have test check object! running test pass...')
        calculateDiscountImpact(open(ck, 'r').read())
        logger.info('test pass done')
    logger.info('leagcy discount system successfully loaded')
