# -*- coding: utf-8 -*-
'''
Created on 03.02.2011

@author: gsu
'''

from sqlalchemy import Column, Integer, String, Numeric, Time, Date, ForeignKey
from sqlalchemy.orm import relation, backref
from simplediscount.connection.connection import Base, Session
from simplediscount.discountrate import DiscountRateMixin
from simplediscount.sprices import Sprices

session = Session()

class AutomaticDiscount(Base, DiscountRateMixin):
    '''
    Автоматическая скидка Штрих-М
    '''

    __tablename__ = 'discountsauto'

    shemecode = Column(Integer, ForeignKey(Sprices.code), primary_key=True)
    disccode = Column(Integer, primary_key=True)
    name = Column(String(30))
    disctype = Column(Integer)
    discmode = Column(Integer)
    discsize = Column(Numeric(precision=15, scale=2))
    rtext = Column(String(30))
    date_beg = Column(Date)
    date_end = Column(Date)
    time_beg = Column(Time(timezone=False))
    time_end = Column(Time(timezone=False))
    days_beg = Column(Integer)
    days_end = Column(Integer)
    quant_beg = Column(Numeric(precision=17, scale=3))
    quant_end = Column(Numeric(precision=17, scale=3))
    sum_beg = Column(Numeric(precision=17, scale=2))
    sum_end = Column(Numeric(precision=17, scale=2))
    pricecode = Column(Integer)
    rsummode = Column(Integer)
    cardgroupcode = Column(Integer)
    birthdaymode = Column(Integer)
    
    catalogid = Column(Integer)
    catalogquantbeg = Column(Numeric(precision=17, scale=3))
    catalogquantend = Column(Numeric(precision=17, scale=3))
    catalogsumbeg = Column(Numeric(precision=15, scale=2))
    catalogsumend = Column(Numeric(precision=15, scale=2))
    
    discval = property(lambda self: self.discsize)

    scheme = relation(Sprices, backref=backref('discounts', order_by='AutomaticDiscount.disccode'))

    def __str__(self):
        return u'Automatic Discount (code: %d, scheme: %d, name: "%s", value: %.2f)' % (self.disccode, self.shemecode, self.name, self.discsize)

    def __repr__(self):
        return str(self)

    def __eq__(self, other):
        return self.shemecode == other.schemecode and self.disccode == other.disccode

    def __hash__(self):
        return hash('%d|%d' % (self.shemecode, self.disccode))
